@@ -43,6 +43,8 @@ sub opt_spec {
       $self->project_id_opt_spec,
       $self->project_workdir_opt_spec,
       $self->file_params_opt_spec,
+      $self->extract_id_from_name_opt_spec,
+      $self->external_tag_opt_spec,
       ;
 
     return @opts;
@@ -64,6 +66,10 @@ sub validate_args {
       if defined $opt->{preset_disassemble_algorithm};
     $self->app->{rundata}->{delete_not_existing} =
       defined $opt->{delete_not_existing} ? $opt->{delete_not_existing} : 0;
+    $self->app->{rundata}->{extract_id_from_name} =
+      defined $opt->{extract_id_from_name} ? $opt->{extract_id_from_name} : 0;
+    $self->app->{rundata}->{external_tag} =
+      defined $opt->{external_tag} ? $opt->{external_tag} : "source:Serge";
 }
 
 sub execute {
@@ -80,12 +86,16 @@ sub execute {
     );
 
     my $project = $app->project_api->get_project;
-    $app->project_api->update_project_external_tag( $project, "source:Serge" ) if ($#{ $project->documents } >= 0);
+    $app->project_api->update_project_external_tag( $project, $rundata->{external_tag} ) if ($#{ $project->documents } >= 0);
     my %documents;
+
     for ( @{ $project->documents } ) {
-        my $key = &get_document_key( $_->full_path, $_->target_language );
-        $documents{$key} = [] unless defined $documents{$key};
-        push @{ $documents{$key} }, $_;
+            my $key = &get_document_key( 
+                $_->full_path,
+                $_->target_language,
+                $rundata->{extract_id_from_name} );
+            $documents{$key} = [] unless defined $documents{$key};
+            push @{ $documents{$key} }, $_;
     }
 
     my %ts_files;
@@ -96,13 +106,19 @@ sub execute {
                 utf8::decode($name); # assume UTF8 filenames
                 utf8::decode($_);
             }
+
             if (   -f $name
                 && !m/^\.$/
                 && m/$rundata->{filetype}$/ )
             {
                 s/$rundata->{filetype}$//;
                 my $path = catfile( dirname($name), $_ );
-                my $key = &get_ts_file_key($rundata->{project_workdir}, $path);
+
+                my $key = &get_ts_file_key(
+                    $rundata->{project_workdir},
+                    $path, 
+                    $rundata->{extract_id_from_name} );
+
                 utf8::decode($key);
                 $ts_files{$key} = [] unless defined $ts_files{$key};
                 push @{ $ts_files{$key} }, $name;
@@ -175,27 +191,25 @@ sub update {
 
     my @target_languages =
       map { &get_language_from_ts_filepath($rundata->{project_workdir}, $_) } @$ts_files;
-    my @project_target_languages = @{ $project->target_languages };
-    my %lang_pairs;
+    my %doc_and_path_by_lang;
     my @files_without_documents;
 
     #print Dumper $ts_files;
     for (@$ts_files) {
 
-        #print $_."\n";
         my $lang = get_language_from_ts_filepath($rundata->{project_workdir}, $_);
         my $doc = first { $_->target_language eq $lang } @$documents;
 
-        #p $doc;
+        # p $doc;
         if ( defined $doc ) {
-            $lang_pairs{$lang} = [ $_, $doc->id ];
+            $doc_and_path_by_lang{$lang} = { path => $_, doc => $doc };
         }
         else {
             push @files_without_documents, $_;
         }
     }
     my @documents_without_files =
-      grep { !exists $lang_pairs{ $_->target_language } } @$documents;
+      grep { !exists $doc_and_path_by_lang{ $_->target_language } } @$documents;
 
     $log->warn(
         "No files for documents:"
@@ -208,7 +222,31 @@ sub update {
         "No documents for files:" . join( ', ', @files_without_documents ) )
       if @files_without_documents;
 
-    $api->update_document( @{ $lang_pairs{$_} } ) for ( keys %lang_pairs );
+    for ( keys %doc_and_path_by_lang ) {
+        my $doc_and_path = $doc_and_path_by_lang{$_};
+        
+        $api->update_document( $doc_and_path->{path}, $doc_and_path->{doc}->id );
+        
+        if ( $rundata->{extract_id_from_name} ) {
+            my $file_name = get_file_name(
+                $doc_and_path->{path},
+                $rundata->{filetype},
+                $target_languages[0]);
+            my $document_name = $doc_and_path->{doc}->name;
+
+            if ($file_name ne $document_name) {
+                $log->info(
+                    sprintf(
+                        "Renaming document '%s' from '%s' to '%s'.",
+                        $doc_and_path->{doc}->id,
+                        $document_name,
+                        $file_name
+                    )
+                );
+                $api->rename_document( $doc_and_path->{doc}->id, $file_name );  
+            }
+        }
+    }
 }
 
 sub _check_if_files_are_empty {
@@ -234,9 +272,16 @@ sub upload {
     croak("Conflict: one target language to one file expected.")
       unless @$ts_files == 1 && @target_languages == 1;
     my $path     = shift @$ts_files;
+
     my $filename = prepare_document_name( $rundata->{project_workdir}, $path, $rundata->{filetype},
         $target_languages[0] );
-    my $documents = $self->app->project_api->upload_file( $path, $filename,
+
+    my $external_id;
+    if ( $rundata->{extract_id_from_name}){
+        $external_id = &get_file_id( $path );
+    }
+
+    my $documents = $self->app->project_api->upload_file( $path, $filename, $external_id,
         \@target_languages );
     $log->info( "Created documents ids:\n  "
           . join( ', ', map { $_->id } @$documents ) );
